% -------------------------------------------------------------------------
%
%     Copyright (C) 2014-2015, Przemyslaw Musialski
%     email     :   pm@cg.tuwien.ac.at
%     created   :   01/10/2015
%     edited    :   10/21/2015
% -------------------------------------------------------------------------
%
%     This code accompanies the paper: 
%
%     P. Musialski, T. Auzinger, M. Birsak, M. Wimmer, and L. Kobbelt, 
%     REDUCED-ORDER SHAPE OPTIMIZATION USING OFFSET SURFACES, 
%     ACM Trans. Graph. (Proc. ACM SIGGRAPH 2015), vol. 34, no. 4, pp. 102:1102:9, Jul. 2015.
% 
%     If you use the code or parts of it in your academic work you need to
%     cite the paper: 
%
%     @article{Musialski2015a,
%     author    = {Musialski, Przemyslaw and Auzinger, Thomas and Birsak, Michael and Wimmer, Michael and Kobbelt, Leif},
%     title     = {{Reduced-Order Shape Optimization Using Offset Surfaces}},
%     journal   = {ACM Transactions on Graphics (Proc. ACM SIGGRAPH 2015)},
%     volume    = {34},
%     year      = {2015}
%     doi       = {10.1145/2766955},
%     issn      = {07300301},
%     month     = jul,
%     number    = {4},
%     pages     = {102:1--102:9},
%     url       = {http://www.cg.tuwien.ac.at/research/publications/2015/musialski-2015-souos/ http://dl.acm.org/citation.cfm?doid=2809654.2766955},
%     }
%
% -------------------------------------------------------------------------
% 
%     This program is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     This program is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% -------------------------------------------------------------------------
%
%     CODE (MATLAB and C/MEX)
%
%     The code has been tested with MATLAB 2014a and 2014b. Additionally,
%     is uses the fmincon function from MATLAB Optimization Toolbox. 
%
%     The included *.cpp files are is currently compiled for x64 with
%     Microsoft Visual C++ 2013 Professional (C)
%     In order to compiled it call:
%     >> mex comp_moments_grad_3d
%     >> mex comp_moments_3d   
%
%     To run an example, execute the script:
        run_quicktest
%     
% -------------------------------------------------------------------------
%
%     ADDITIONAL EXAMPLES 
%     
%     Additional datasets are contained in the separate archive 'data.zip'.
%     Place the content of the archive in the root folder as
%     ./data/
%     
%     To create the examples, just run the scripts:      
%       run_bottle
%       run_box      
%       run_kitten
%       run_rabbit
%       run_seaturtle
%       run_fish
%       run_armadillo        
%
% -------------------------------------------------------------------------
%
%     SKELETON COMPUTATION / MESH CONTRACTION
%
%     For the computation of MCF-Skeletons as proposed in: 
%     Tagliasacchi, A., Alhashim, I., Olson, M., and Zhang, H. 2012. 
%     Mean Curvature Skeletons. Computer Graphics Forum 31, 5, 17351744.     
%     the implementation available on CGAL has been used: 
%     http://doc.cgal.org/4.7/Surface_mesh_skeletonization/index.html
%     This code is not included in this package, however, the skeleton
%     file for the armadillo model is available in the ./data/contraction_mfc
%     folder. 
%     
%     All other skeletons have been computed with a contraction method based
%     on the paper:
%     Au, O.K.-C., Tai, C.-L., Chu, H.-K., Cohen-Or, D., and Lee, T.-Y. 2008. 
%     Skeleton extraction by mesh contraction. ACM Transactions on Graphics 27, 3, 1.
%     
%     However, note that this implementation does not always deliver valid
%     skeletons (see file: comp_max_offset_3d.m). %      
% 
% -------------------------------------------------------------------------
%
%     THIRD PARTY SOFTWARE USED IN THIS PROJECT
%
%     This package contains the following functions from the "gptoolbox" 
%     authored by Alec Jacobson: https://github.com/alecjacobson/gptoolbox
%     	- cotmatrix.m
% 		- massmatrix.m
% 		- massmatrix_intrinsic.m 
%
%     The function
%       - plotMesh is from the MATLAB geometry package of Ryan Schmidt: 
%         http://www.dgp.toronto.edu/~rms/software/matlabmesh/
%
%     This package also used the LIBIGL library: https://github.com/libigl/libigl
%     which is not contained. In order to compile the mex-wrappers
%     	- readOBJ_mex.cpp
%     	- writeOBJ_mex.cpp
%     	- readOFF_mex.cpp
%     	- writeOFF_mex.cpp
%     you will need to place LIBIGL and the EIGEN library into the ./lib/ folder.
%     For compilation refer to the file compileMEX.m. The current binaries 
%     are compiled for x64 with Microsoft Visual C++ 2013 Professional (C)  
%
% -------------------------------------------------------------------------




