function [ s ] = comp_combined_offsetSurface3d( filename, varargin )
%     %--- save the resulting meshes?
%     save_text = 1;
%     save_mesh = 0;  
%
%     %--- inner_bounds
%     %max_bound = 'contraction';
%     %max_bound = 'mfc';
%
%     %--- min-max bounds
%     min_inside = 0.01;
%     max_inside = 0.1;  
%     min_outside = 0;
%     max_outside = 0.1;
%     bounds = [ min_inside, max_inside, min_outside, max_outside ];
%     cut_y  = [ cut_low ; cut_upp ];
%
%     %--- eigen-bases
%     basis_k = 36;
%     lap_type = 'cotan'
%
%     %--- solver choices
%     %solver = 'interior-point';
%     %solver = 'sqp';
%     %solver = 'active-set';
%     %display = 'iter-detailed','final','notify','off';
% 
%     %--- costfunction type
%     %costfun = 'none'; % no solving, input = output
%     %costfun = 1; % alpha, tets, finite diff
%     %costfun = 'findiff'; % alpha, divergence, finite diff
%     %costfun = 'default'; % alpha, divergence, own grad
% 
%     %--- approach
%     %approach = 1; % single displacement to the interior
%     %approach = 2; % double displacement to the interior and exterior
%
%     %--- gradient: 'on' = own grad, 'off' = fin-diff      
% 
%     %--- weights
%     weights = [ 1.0, 1.0, 1.0, 0.0, 1.0, 1e-1 ];
%
% -------------------------------------------------------------------------
% copyright :   (c) 2014-2015 Przemyslaw Musialski
% email     :   pm@cg.tuwien.ac.at
% created   :   01/10/2015
% edited    :   05/05/2015
% -------------------------------------------------------------------------

%% parse input
   p = inputParser;   
   addRequired(p,'filename',@ischar);
   addOptional(p,'folder','output');   
   addOptional(p,'folderin','./data/');   
   addOptional(p,'save_text',0);
   addOptional(p,'save_mesh',0);   
   addOptional(p,'max_bound','contraction');  
   addOptional(p,'bounds',[ 0.0, 0.1, 0.05, 0.95 ]);
   addOptional(p,'basis_k',36);
   addOptional(p,'lap_type','cotan');
   addOptional(p,'lap_weight','simple');   
   addOptional(p,'solver','active-set'); % active-set
   addOptional(p,'display','iter-detailed');  %iter, final  
   addOptional(p,'costfun',2);
   addOptional(p,'approach',1);
   addOptional(p,'constoffset',[0,0]); % this is a fixed offset over the entire surface (works only on one-sided offsets)   
   addOptional(p,'cutplanesi',[  ]); % lower and upper y-planes, where the offset is fixed below and above
   addOptional(p,'cutplaneso',[  ]); % lower and upper y-planes, where the offset is fixed below and above
   addOptional(p,'cutvali',[ 0.0, 0.0 ]); % values for fixed displacement
   addOptional(p,'cutvalo',[ 0.0, 0.0 ]); % values for fixed displacement
   addOptional(p,'fixpointsi',[  ]); % vector of indices for fixed vertices (replaces the cut-plance or combines with them?)
   addOptional(p,'fixvaluesi',[  ]); % vector of values for individual fixed offsets
   addOptional(p,'fixpointso',[  ]); % vector of indices for fixed vertices (replaces the cut-plance or combines with them?)
   addOptional(p,'fixvalueso',[  ]); % vector of values for individual fixed offsets
   addOptional(p,'bottomplane',[ 0, 0.0 ]); % disallow displacements below the second value (e.g., ground-plane)
   addOptional(p,'usenormals',[0,0]); % use [inner,outer] normals
   addOptional(p,'shiftY', 0); % shift the model to have 0-y coordinate
   addOptional(p,'gradient','off');
   addOptional(p,'comment','no comment');
   addOptional(p,'rho',[1.0 1.0]);
   addOptional(p,'radii',[1.0 1.0]);
   addOptional(p,'weights',[ 1.0, 1.0, 1.0, 0.0, 1.0, 1e-1 ]);
   

   parse(p,filename,varargin{:});
   args = p.Results;
   

%%
disp('---- Loading mesh ...');
meshfile = [args.folderin, filename, '.obj'];
if exist(meshfile, 'file')
    tic
    s.m1 = readObj( meshfile );
    s.filename = filename;
    toc
else
    error('File "%s" does not exist. Exiting.', filename);    
end


%% compute maximal inner offsets and other preprocessing stuff

filecontracted = [args.folderin,'contraction/', filename, '_cont_i.obj'];
filecorr = [args.folderin,'contraction_mfc/', filename, '_meso_corr_vertices.txt'];

s.stype = args.max_bound;
if strcmp(args.max_bound,'contraction')   
    if exist(filecontracted, 'file')
        mi = readObj( filecontracted );
        dvec_max = mi.v-s.m1.v;
        d_max = sqrt(sum(dvec_max.*dvec_max,2));
    else        
        [ d_max, dvec_max ] = comp_max_offset_3d( s.m1, s.stype, 1000, 1e-6, 1.1, 1.0 ); 
    end
elseif strcmp(args.max_bound,'mcf')
    if exist(filecorr,'file')
        % these contraction skeleton are computed with the algorithm of:
        % > Tagliasacchi, A., Alhashim, I., Olson, M., and Zhang, H. 2012. 
        % > Mean Curvature Skeletons. Computer Graphics Forum 31, 5, 17351744.
        
        VC = load([args.folderin,'contraction_mfc/' filename, '_meso_corr_vertices.txt']);
        dvec_max = VC(:,4:6)-VC(:,1:3);
        d_max = sqrt(sum(dvec_max.*dvec_max,2));
        fprintf('Min d: %f, max d: %f\n', min(d_max), max(d_max));
    else
        error('File "%s" does not exist. Exiting.', filecorr);
    end
else
    [ d_max, dvec_max ] = comp_max_offset_3d( s.m1, s.stype );
end

% get displacement bounds
s.mindi     = args.bounds(1);
s.maxdi     = args.bounds(2);
s.mindo     = args.bounds(3);
s.maxdo     = args.bounds(4);

% normalize contraction vector field
dvec_max_n  = bsxfun(@rdivide, dvec_max, sqrt(dot(dvec_max,dvec_max,2)));

% inner surface bounds (lower and upper)
if args.usenormals(1)==1
    s.m1.din    = -s.m1.n;
    s.diub      = s.maxdi.*ones(length(s.m1.v),1);
    s.dilb      = s.mindi.*ones(length(s.m1.v),1);
else
    % inner surface bounds (lower and upper)    
    s.m1.din    = +dvec_max_n;        
    % project di_min from normals to vectors v
    proj        = abs(dot(+dvec_max_n,-s.m1.n,2));
    % clamp proj to cos(60degree) = 0.5
    proj(proj<0.5)=0.5;
    
    % scale to desired di_min
    dilb        = s.mindi./proj;
    
    % assign to shape struct
    s.dilb      = dilb;
    
    % -- upper bounds are 0.95 * the length of the v-vector
    s.diub      = s.maxdi*d_max;
end

% outer surface bounds
if args.usenormals(2)==1 % use surface normals for outer displacement (default off)
    s.m1.don    = +s.m1.n;
    s.doub      = s.maxdo.*ones(length(s.m1.v),1);
    s.dolb       = s.mindo.*ones(length(s.m1.v),1);
else % use the inverted contraction vectors
    % outer surface bounds
    s.m1.don    = -dvec_max_n;
        
    % outer surface bounds (lower and upper)    
    s.m1.don    = -dvec_max_n;        
    % project di_min from normals to vectors v
    proj        = abs(dot(-dvec_max_n,+s.m1.n,2));
    % clamp proj to cos(60degree) = 0.5
    proj(proj<0.5)=0.5;
    % scale to desired di_min
    doub        = s.maxdo./proj;
    dolb        = s.mindo./proj;    
    s.doub      = doub;
    s.dolb      = dolb;
end;


%% cut planes and fixed values
s.m1.fixpointsi = [];
s.m1.fixvaluesi = [];
s.m1.fixpointso = [];
s.m1.fixvalueso = [];

% inner fixe points
if ~isempty(args.fixpointsi)
    % additinoal fixed points provides by used:
    s.m1.fixpointsi = args.fixpointsi;    
    s.m1.fixvaluesi = args.fixvaluesi;  
    
    % hack: fix normals
    %s.m1.din(s.m1.fixpointsi)    = -s.m1.n(s.m1.fixpointsi);
end

% outer fix points
if ~isempty(args.fixpointso)
    s.m1.fixpointso = args.fixpointso;
    s.m1.fixvalueso = args.fixvalueso;
    % hack: fix normals
    s.m1.don(s.m1.fixpointso)    = +s.m1.n(s.m1.fixpointso);
    s.dolb(s.m1.fixpointso)      = s.mindo;
    s.doub(s.m1.fixpointso)      = s.maxdo;
end
% cut-plane thresholds, i.e., create constrained vertices below and above
% for inner surface
if ~isempty(args.cutplanesi)
    cutloweri = (s.m1.v(:,2)<=args.cutplanesi(1)); % lower cut-off plane inner 
    cutupperi = (s.m1.v(:,2)>=args.cutplanesi(2)); % upper cut-off plane inner 
    % get indices
    lower = find(cutloweri);
    upper = find(cutupperi);
    s.m1.fixpointsi = [ s.m1.fixpointsi; lower ; upper ];
    s.m1.fixvaluesi = [ s.m1.fixvaluesi; args.cutvali(1)*ones(size(lower)); args.cutvali(2)*ones(size(upper)) ];
end
% cut-plane thresholds, i.e., create constrained vertices below and above
% for outer surface
if ~isempty(args.cutplaneso)
    cutlowero = (s.m1.v(:,2)<=args.cutplaneso(1)); % lower cut-off plane outer
    cutuppero = (s.m1.v(:,2)>=args.cutplaneso(2)); % upper cut-off planeouter
    % get indices
    lower = find(cutlowero);
    upper = find(cutuppero);
    s.m1.fixpointso = [ s.m1.fixpointso; lower ; upper ];
    s.m1.fixvalueso = [ s.m1.fixvalueso; args.cutvalo(1)*ones(size(lower)); args.cutvalo(2)*ones(size(upper)) ];
end

%% apply fixed offsets in direction opposite to the varying offset
s.m1.dfi    = args.constoffset(1)*ones(length(s.m1.v),1);
s.m1.dfo    = args.constoffset(2)*ones(length(s.m1.v),1);

% add additional constrains and values
s.m1.R2 = args.radii(1)^2;
s.m1.radii = args.radii;

% backup mesh
s.m0 = s.m1;

%% compute Laplacian and Eigen-bases

disp('---- Processing mesh ...');
% number eigen-bases
s.k     = args.basis_k;
s.Ltype = args.lap_type;
tic
[ L, G, D ] = comp_laplacian3d_eig( s.m1, s.k, args.lap_type, 0, args.lap_weight );
t_prep  = toc;
s.dv    = L*s.m1.v;
s.H     = sqrt(sum(s.dv .* s.dv,2))/2;
s.G     = G;
s.L     = L;
s.D     = D;


%% solve
disp('---- Solving ...');
solver = args.solver;
display = args.display;
costfun = args.costfun; 
approach = args.approach; 
weights = args.weights;

% scale for alpha0
s.a0 = 1e-4;

% copy mesh
s.m1 = s.m0;
s.m2 = s.m0;

if approach==1 %'single inside'
        
    alpha0 = G\(s.dilb+0.1*s.diub);        
    bounds = [ s.dilb, s.diub, s.dolb, s.doub ]; 
    
    [ di, ~, result, exitflag, output, time, fhist, fmap ] = solve_combined_offsets3d_fmincon( s.m1, costfun, weights, G, L, alpha0, args.basis_k, bounds, solver, args.gradient, args.rho, approach, display);  
    
    % variable surface
    s.m2.v(:,1) = s.m2.v(:,1) + di .* s.m1.din(:,1);
    s.m2.v(:,2) = s.m2.v(:,2) + di .* s.m1.din(:,2);
    s.m2.v(:,3) = s.m2.v(:,3) + di .* s.m1.din(:,3);
    
    % fixed surface
    s.m1.v(:,1) = s.m1.v(:,1) + s.m1.dfo .* s.m1.don(:,1);
    s.m1.v(:,2) = s.m1.v(:,2) + s.m1.dfo .* s.m1.don(:,2);
    s.m1.v(:,3) = s.m1.v(:,3) + s.m1.dfo .* s.m1.don(:,3);   
    
elseif approach==2 %'double'
    
    % use a twice as big alpha0 for both direction displacement
    alpha0 = [G\(s.dilb+0.1*s.diub); G\(s.dolb+0.1*s.doub)];        
    bounds = [ s.dilb, s.diub, s.dolb, s.doub ]; 
    
    [ di, do, result, exitflag, output, time, fhist, fmap ] = solve_combined_offsets3d_fmincon( s.m1, costfun, weights, G, L, alpha0, args.basis_k, bounds, solver, args.gradient, args.rho, approach, display);    
            
    s.m2.v(:,1) = s.m2.v(:,1) + di .* s.m1.din(:,1);
    s.m2.v(:,2) = s.m2.v(:,2) + di .* s.m1.din(:,2);
    s.m2.v(:,3) = s.m2.v(:,3) + di .* s.m1.din(:,3);
    
    s.m1.v(:,1) = s.m1.v(:,1) + do .* s.m1.don(:,1);
    s.m1.v(:,2) = s.m1.v(:,2) + do .* s.m1.don(:,2);
    s.m1.v(:,3) = s.m1.v(:,3) + do .* s.m1.don(:,3);    
        
    %figure(1), plot_shape3d( s, 0, output  );
    
elseif approach==3 %'single outside'
        
    alpha0 = G\(s.dilb+0.1*s.diub);        
    bounds = [ s.dilb, s.diub, s.dolb, s.doub ]; 
    
    [ ~, do, result, exitflag, output, time, fhist, fmap ] = solve_combined_offsets3d_fmincon( s.m1, costfun, weights, G, L, alpha0, args.basis_k, bounds, solver, args.gradient, args.rho, approach, display);  
    
    % variable surface
    s.m1.v(:,1) = s.m1.v(:,1) + do .* s.m1.don(:,1);
    s.m1.v(:,2) = s.m1.v(:,2) + do .* s.m1.don(:,2);
    s.m1.v(:,3) = s.m1.v(:,3) + do .* s.m1.don(:,3);  
            
    % fixed surface
    s.m2.v(:,1) = s.m2.v(:,1) + s.m1.dfi .* s.m1.din(:,1);
    s.m2.v(:,2) = s.m2.v(:,2) + s.m1.dfi .* s.m1.din(:,2);
    s.m2.v(:,3) = s.m2.v(:,3) + s.m1.dfi .* s.m1.din(:,3);                            
    
elseif approach==4 % 'full solution inside (extremly slow and might not converge on large meshes)'
    
    delta0 = s.dilb+0.1*s.diub;
    delta_k = length(delta0);
    bounds = [ s.dilb, s.diub, s.dolb, s.doub ]; 
    
    [ di, ~, result, exitflag, output, time, fhist, fmap ] = solve_full_offsets3d_fmincon( s.m1, costfun, weights, L, delta0, delta_k, bounds, solver, args.gradient, args.rho, approach, display);  
    
    % variable surface
    s.m2.v(:,1) = s.m2.v(:,1) + di .* s.m1.din(:,1);
    s.m2.v(:,2) = s.m2.v(:,2) + di .* s.m1.din(:,2);
    s.m2.v(:,3) = s.m2.v(:,3) + di .* s.m1.din(:,3);
    
    % fixed surface
    s.m1.v(:,1) = s.m1.v(:,1) + s.m1.dfo .* s.m1.don(:,1);
    s.m1.v(:,2) = s.m1.v(:,2) + s.m1.dfo .* s.m1.don(:,2);
    s.m1.v(:,3) = s.m1.v(:,3) + s.m1.dfo .* s.m1.don(:,3);  
else
    error('wrong approach ;)');
end


%% save
datestring = datestr(now, 'yyyy-mm-dd-HH-MM-SS');
if ~exist([args.folder], 'dir')
    mkdir(args.folder);
end
meshname = [args.folder, '/', args.filename, '_' ,int2str(args.approach), '_', args.costfun, '_' args.comment];        
s.meshname = meshname;

%% compute moments of the body
if 1 %strcmp(args.costfun,'bottle') || strcmp(args.costfun,'none') || strcmp(args.costfun,'stand')
    % save moments of of both bodies: shell and interrior
    % 1. get the moments of inner mesh
    
    [M2,~,~, ~, ~, ~] = comp_inertia_3d( (s.m2), [] );    
    [P_shell,~,Iabc12, ~, ~, ~] = comp_inertia_3d( s.m1, (s.m2) );    
    [P_out,~,Iabc1, ~, ~, ~] = comp_inertia_3d( s.m1, [] );    
    [M0,~,Iabc0, ~, ~, ~] = comp_inertia_3d( s.m0, [] );
    P_shell(1) = args.rho(1)*P_shell(1);
    M2(1) = args.rho(2)*M2(1);
    mm1 = P_shell(1)+M2(1);
    % optmize the joint COM of both
    c2x = 0*P_shell(2) + ((P_shell(1)*P_shell(2) + M2(1)*M2(2))/mm1);
    c2y = 0*P_shell(3) + ((P_shell(1)*P_shell(3) + M2(1)*M2(3))/mm1);
    c2z = 0*P_shell(4) + ((P_shell(1)*P_shell(4) + M2(1)*M2(4))/mm1);
    
    Moments2 = [ Iabc1'; [c2x, c2y, c2z] ];
    dlmwrite([meshname, '_optim_storage.dlm'], Moments2);
    
    Moments_outer = [ Iabc1'; [P_out(2), P_out(3), P_out(4)] ];
    dlmwrite([meshname, '_optim_out.dlm'], Moments_outer);
    
    Moments_shell = [ Iabc12'; [P_shell(2), P_shell(3), P_shell(4)] ];
    dlmwrite([meshname, '_optim_shell.dlm'], Moments_shell);
    
    Moments0 = [ Iabc0'; [M0(2), M0(3), M0(4)] ];
    dlmwrite([meshname, '_optim_original.dlm'], Moments0);
end


% compute final moments of the solid shell
% compute tensor of inertia
[M,I,P, ~, WS, WI] = comp_inertia_3d( s.m1, s.m2 );
s.S = M;
s.P = P;
s.I = I;
s.WS = WS;
s.WI = WI;
Moments = [ s.P'; s.S(2:4)' ];
dlmwrite([meshname, '.dlm'], Moments);
            
if args.save_mesh 
    % write mesh   
    disp('---- Write mesh ...');
    
    % flip the inner mesh - now its normals show to the inside!    
    % write both meshes
    writeOBJ_mex([meshname, '_o.obj'], s.m1.v, s.m1.f);
    mf2 = flipMesh(s.m2);
    writeOBJ_mex([meshname, '_i.obj'], mf2.v, mf2.f);
    %writeOBJ_mex([meshname, '_i.obj'], s.m2.v, s.m2.f);
        
    disp('---- Done.');
end

%% save some numbers
if args.save_text        
    Io = result(5:7);

    fileID = fopen([args.folder, '/_', args.filename, '_exp1.txt'],'a');
    fprintf(fileID,'--- - - - %s\n', datestring);
    fprintf(fileID,'[ %s ]\n', args.comment);
    fprintf(fileID,'file: %s | Lap: %s | LapW: %s | k:%d | approach: %d | solver: %s | costfun: %s | gradient: %s | ', s.filename, args.lap_type, args.lap_weight, s.k, approach, solver, costfun, args.gradient);
    fprintf(fileID,'w: [ %2.2f, %2.2f, %2.2f, %2.2f, %2.2f, %2.2f ] \n', args.weights);
    fprintf(fileID,'b: [ %1.2f, %1.2f, %1.2f, %1.2f ] | fix: [ %1.2f, %1.2f ], ', args.bounds, args.constoffset);
    fprintf(fileID,'a0: %1.6f | exitflag: %d, iter: %d, t_prep: %3.4f, t_solv: %3.4f \n', s.a0, exitflag, output.iterations, t_prep, time);
    fprintf(fileID,'f: %3.4f | fr: %3.4f || result: %2.4f | %2.4f, %2.4f, %2.4f | %2.4f, %2.4f, %2.4f | %2.4f, %2.4f, %2.4f || ', fmap('Etot'), fmap('Ered'), result );
    fprintf(fileID,'%2.4f, %2.4f || %2.4f, %2.4f ||\n', result(5)/result(6), result(7)/result(6), abs(result(5)-result(6)), abs(result(7)-result(6)) );
    %fprintf(fileID,'M: %2.4f | %2.4f, %2.4f, %2.4f | %2.4f, %2.4f, %2.4f | %2.4f, %2.4f, %2.4f', M);   
        
    if strcmp(args.costfun,'fish')
        %fprintf(fileID,'sqrt(f2): %4.4f, || Vout: %2.4f, Vin: %2.4f, Vshell: %2.4f \n\n', s.m1.sqrtf2, s.m1.volumes);
        
        [P_shell,~,~, ~, ~, ~] = comp_inertia_3d( s.m1, (s.m2) );    
        [P_out,~,~, ~, ~, ~] = comp_inertia_3d( s.m1, [] );    
        [P_in,~,~, ~, ~, ~] = comp_inertia_3d( s.m2, [] );    
        f2 = (args.rho(2)*P_out(1)-args.rho(1)*P_shell(1))^2;
        com_out  = P_out(2:4);
        com_shell = P_shell(2:4);
        
        fprintf(fileID,'sqrt(f2): %4.4f | diff. com: %2.4f, %2.4f, %2.4f | \n', sqrt(f2), sqrt((com_shell-com_out).^2));
        fprintf(fileID,'V_out: %2.4f, V_in: %2.4f, V_shell: %2.4f \n\n', P_out(1), P_in(1), P_shell(1));
        
    end
    fclose(fileID);
    
    % Write CVS Table containing all the numbers needed to create a diagram
    % in Excel
    table_path = [args.folder, '/_', args.filename, '_table1.csv'];
    table_exists = 0;
    if exist(table_path, 'file')
        table_exists = 1;
    end
    
    fileID2 = fopen(table_path,'a');
    %fprintf(fileID,'--- - - - %s\n', datestring);
    %fprintf(fileID,'[ %s ]\n', args.comment);
    if table_exists == 0
        fprintf(fileID2,'filename;lap type;lap weight;k;approach;solver;costfun;gradient;weights;;;;;;bounds;;;;const offset;;a0;exitflag;iter;t_prep;t_solve;f;result;;;;;;;;;;;;;;stable with w<\n');
    end
    fprintf(fileID2,'%s;%s;%s;%d;%d;%s;%s;%s;', s.filename, args.lap_type, args.lap_weight, s.k, approach, solver, costfun, args.gradient);
    fprintf(fileID2,'%2.2f;%2.2f;%2.2f;%2.2f;%2.2f;%2.2f;', args.weights);
    fprintf(fileID2,'%1.2f;%1.2f;%1.2f;%1.2f;%1.2f;%1.2f;', args.bounds, args.constoffset);
    fprintf(fileID2,'%1.6f;%d;%d;%3.4f;%3.4f;', s.a0, exitflag, output.iterations, t_prep, time);
    fprintf(fileID2,'%3.4f;%2.4f;%2.4f;%2.4f;%2.4f;%2.4f;%2.4f;%2.4f;%2.4f;%2.4f;%2.4f;', fhist(end), result );
    fprintf(fileID2,'%2.4f;%2.4f;%2.4f;%2.4f\n', result(5)/result(6), result(7)/result(6), abs(result(5)-result(6)), abs(result(7)-result(6)) );
    fclose(fileID2);
    
end 

s.args = args;



