function [ mesh ] = readObj( filename )
% initialize mesh data structure from vertex/face/normal/uv-arrays
%
% mesh.v = Nx3 list of vertices
% mesh.n = Nx3 list of vertex normals
% mesh.u = Nx2 list of vertex uv's
% mesh.f = Mx3 list of triangles
% mesh.fn = Mx3 list of triangle face normals
% mesh.vidx = Nx1 list of vertex indices (useful for various things)
% mesh.fidx = Nx1 list of face indices (useful for various things)
% mesh.bounds = [min;max;center] bounding box
% mesh.e = symmetric sparse array of edges
%             mesh.e(i,j) = 1 if boundary edge, 2 if interior edge
% mesh.te = sparse array of edge triangles
%             mesh.e(i,j) = tri1, mesh.e(j,i) = tri2
%             for boundary tri, one of these will be 0...
% mesh.loops = boundary loops of mesh
% mesh.valence = valence of each vertex of mesh
% mesh.isboundaryv = 0/1 flags for boundary vertices
% mesh.isboundaryt = 0/1 flags for boundary triangles
%
%   [Ryan Schmidt       rms@dgp.toronto.edu     09/2008]
%   [Przem Musialski    pm@cg.tuwien.ac.at      12/2014]

mesh = struct('v', [], 'n', [], 'u', [], 'f', [], 'e', [], 'bounds', [], 'loops', []);

[V,F,VN,FN] = readOBJ_mex(filename);
mesh.v = V;
mesh.f = F;
mesh.n = VN;
mesh.fn = FN;

nVerts = size(mesh.v,1);
nFaces = size(mesh.f,1);
mesh.vidx = (1:nVerts)';
mesh.fidx = (1:nFaces)';

% compute mesh bounds -  min, max, center
mesh.bounds = [ min(mesh.v) ; max(mesh.v) ; 0.5*(min(mesh.v) + max(mesh.v)) ];
%mesh = flipMesh(mesh);
end

